@@ -14,6 +14,8 @@
 
 #include "osd_types.h"
 #include "include/ceph_features.h"
+#include "PG.h"
+#include "OSDMap.h"
 
 // -- osd_reqid_t --
 void osd_reqid_t::encode(bufferlist &bl) const
@@ -51,8 +53,9 @@ void osd_reqid_t::generate_test_instances(list<osd_reqid_t*>& o)
 
 void object_locator_t::encode(bufferlist& bl) const
 {
-  ENCODE_START(3, 3, bl);
+  ENCODE_START(4, 3, bl);
   ::encode(pool, bl);
+  int32_t preferred = -1;  // tell old code there is no preferred osd (-1).
   ::encode(preferred, bl);
   ::encode(key, bl);
   ENCODE_FINISH(bl);
@@ -60,16 +63,16 @@ void object_locator_t::encode(bufferlist& bl) const
 
 void object_locator_t::decode(bufferlist::iterator& p)
 {
-  DECODE_START_LEGACY_COMPAT_LEN(3, 3, 3, p);
+  DECODE_START_LEGACY_COMPAT_LEN(4, 3, 3, p);
   if (struct_v < 2) {
     int32_t op;
     ::decode(op, p);
     pool = op;
     int16_t pref;
     ::decode(pref, p);
-    preferred = pref;
   } else {
     ::decode(pool, p);
+    int32_t preferred;
     ::decode(preferred, p);
   }
   ::decode(key, p);
@@ -79,7 +82,6 @@ void object_locator_t::decode(bufferlist::iterator& p)
 void object_locator_t::dump(Formatter *f) const
 {
   f->dump_int("pool", pool);
-  f->dump_int("preferred", preferred);
   f->dump_string("key", key);
 }
 
@@ -87,9 +89,8 @@ void object_locator_t::generate_test_instances(list<object_locator_t*>& o)
 {
   o.push_back(new object_locator_t);
   o.push_back(new object_locator_t(123));
-  o.push_back(new object_locator_t(123, 456));
-  o.push_back(new object_locator_t(1234, -1, "key"));
-  o.push_back(new object_locator_t(12, 789, "key2"));
+  o.push_back(new object_locator_t(1234, "key"));
+  o.push_back(new object_locator_t(12, "key2"));
 }
 
 
@@ -180,6 +181,46 @@ bool pg_t::parse(const char *s)
   return true;
 }
 
+bool pg_t::is_split(unsigned old_pg_num, unsigned new_pg_num, set<pg_t> *children) const
+{
+  assert(m_seed < old_pg_num);
+  if (new_pg_num <= old_pg_num)
+    return false;
+
+  bool split = false;
+  if (true) {
+    int old_bits = pg_pool_t::calc_bits_of(old_pg_num);
+    int old_mask = (1 << old_bits) - 1;
+    for (int n = 1; ; n++) {
+      int next_bit = (n << (old_bits-1));
+      unsigned s = next_bit | m_seed;
+
+      if (s < old_pg_num || s == m_seed)
+	continue;
+      if (s >= new_pg_num)
+	break;
+      if ((unsigned)ceph_stable_mod(s, old_pg_num, old_mask) == m_seed) {
+	split = true;
+	if (children)
+	  children->insert(pg_t(s, m_pool, m_preferred));
+      }
+    }
+  }
+  if (false) {
+    // brute force
+    int old_bits = pg_pool_t::calc_bits_of(old_pg_num);
+    int old_mask = (1 << old_bits) - 1;
+    for (unsigned x = old_pg_num; x < new_pg_num; ++x) {
+      unsigned o = ceph_stable_mod(x, old_pg_num, old_mask);
+      if (o == m_seed) {
+	split = true;
+	children->insert(pg_t(x, m_pool, m_preferred));
+      }
+    }
+  }
+  return split;
+}
+
 void pg_t::dump(Formatter *f) const
 {
   f->dump_unsigned("pool", m_pool);
@@ -210,6 +251,19 @@ ostream& operator<<(ostream& out, const pg_t &pg)
 
 // -- coll_t --
 
+bool coll_t::is_temp(pg_t& pgid) const
+{
+  const char *cstr(str.c_str());
+  if (!pgid.parse(cstr))
+    return false;
+  const char *tmp_start = strchr(cstr, '_');
+  if (!tmp_start)
+    return false;
+  if (strncmp(tmp_start, "_TEMP", 5) == 0)
+    return true;
+  return false;
+}
+
 bool coll_t::is_pg(pg_t& pgid, snapid_t& snap) const
 {
   const char *cstr(str.c_str());
@@ -320,6 +374,8 @@ std::string pg_state_string(int state)
     oss << "active+";
   if (state & PG_STATE_CLEAN)
     oss << "clean+";
+  if (state & PG_STATE_RECOVERING)
+    oss << "recovering+";
   if (state & PG_STATE_DOWN)
     oss << "down+";
   if (state & PG_STATE_REPLAY)
@@ -330,6 +386,8 @@ std::string pg_state_string(int state)
     oss << "splitting+";
   if (state & PG_STATE_DEGRADED)
     oss << "degraded+";
+  if (state & PG_STATE_REMAPPED)
+    oss << "remapped+";
   if (state & PG_STATE_SCRUBBING)
     oss << "scrubbing+";
   if (state & PG_STATE_SCRUBQ)
@@ -401,8 +459,6 @@ void pg_pool_t::dump(Formatter *f) const
   f->dump_int("object_hash", get_object_hash());
   f->dump_int("pg_num", get_pg_num());
   f->dump_int("pg_placement_num", get_pgp_num());
-  f->dump_int("localized_pg_num", get_lpg_num());
-  f->dump_int("localized_pg_placement_num", get_lpgp_num());
   f->dump_unsigned("crash_replay_interval", get_crash_replay_interval());
   f->dump_stream("last_change") << get_last_change();
   f->dump_unsigned("auid", get_auid());
@@ -434,8 +490,6 @@ void pg_pool_t::calc_pg_masks()
 {
   pg_num_mask = (1 << calc_bits_of(pg_num-1)) - 1;
   pgp_num_mask = (1 << calc_bits_of(pgp_num-1)) - 1;
-  lpg_num_mask = (1 << calc_bits_of(lpg_num-1)) - 1;
-  lpgp_num_mask = (1 << calc_bits_of(lpgp_num-1)) - 1;
 }
 
 /*
@@ -452,6 +506,11 @@ bool pg_pool_t::is_pool_snaps_mode() const
   return removed_snaps.empty() && get_snap_seq() > 0;
 }
 
+bool pg_pool_t::is_unmanaged_snaps_mode() const
+{
+  return removed_snaps.size() && get_snap_seq() > 0;
+}
+
 bool pg_pool_t::is_removed_snap(snapid_t s) const
 {
   if (is_pool_snaps_mode())
@@ -488,7 +547,7 @@ snapid_t pg_pool_t::snap_exists(const char *s) const
 
 void pg_pool_t::add_snap(const char *n, utime_t stamp)
 {
-  assert(removed_snaps.empty());
+  assert(!is_unmanaged_snaps_mode());
   snapid_t s = get_snap_seq() + 1;
   snap_seq = s;
   snaps[s].snapid = s;
@@ -499,7 +558,7 @@ void pg_pool_t::add_snap(const char *n, utime_t stamp)
 void pg_pool_t::add_unmanaged_snap(uint64_t& snapid)
 {
   if (removed_snaps.empty()) {
-    assert(snaps.empty());
+    assert(!is_pool_snaps_mode());
     removed_snaps.insert(snapid_t(1));
     snap_seq = 1;
   }
@@ -515,7 +574,7 @@ void pg_pool_t::remove_snap(snapid_t s)
 
 void pg_pool_t::remove_unmanaged_snap(snapid_t s)
 {
-  assert(snaps.empty());
+  assert(is_unmanaged_snaps_mode());
   removed_snaps.insert(s);
   snap_seq = snap_seq + 1;
   removed_snaps.insert(get_snap_seq());
@@ -537,10 +596,7 @@ SnapContext pg_pool_t::get_snap_context() const
  */
 pg_t pg_pool_t::raw_pg_to_pg(pg_t pg) const
 {
-  if (pg.preferred() >= 0 && lpg_num)
-    pg.set_ps(ceph_stable_mod(pg.ps(), lpg_num, lpg_num_mask));
-  else
-    pg.set_ps(ceph_stable_mod(pg.ps(), pg_num, pg_num_mask));
+  pg.set_ps(ceph_stable_mod(pg.ps(), pg_num, pg_num_mask));
   return pg;
 }
   
@@ -551,10 +607,7 @@ pg_t pg_pool_t::raw_pg_to_pg(pg_t pg) const
  */
 ps_t pg_pool_t::raw_pg_to_pps(pg_t pg) const
 {
-  if (pg.preferred() >= 0 && lpgp_num)
-    return ceph_stable_mod(pg.ps(), lpgp_num, lpgp_num_mask) + pg.pool();
-  else
-    return ceph_stable_mod(pg.ps(), pgp_num, pgp_num_mask) + pg.pool();
+  return ceph_stable_mod(pg.ps(), pgp_num, pgp_num_mask) + pg.pool();
 }
 
 void pg_pool_t::encode(bufferlist& bl, uint64_t features) const
@@ -569,6 +622,7 @@ void pg_pool_t::encode(bufferlist& bl, uint64_t features) const
     ::encode(object_hash, bl);
     ::encode(pg_num, bl);
     ::encode(pgp_num, bl);
+    __u32 lpg_num = 0, lpgp_num = 0;  // tell old code that there are no localized pgs.
     ::encode(lpg_num, bl);
     ::encode(lpgp_num, bl);
     ::encode(last_change, bl);
@@ -596,6 +650,7 @@ void pg_pool_t::encode(bufferlist& bl, uint64_t features) const
     ::encode(object_hash, bl);
     ::encode(pg_num, bl);
     ::encode(pgp_num, bl);
+    __u32 lpg_num = 0, lpgp_num = 0;  // tell old code that there are no localized pgs.
     ::encode(lpg_num, bl);
     ::encode(lpgp_num, bl);
     ::encode(last_change, bl);
@@ -609,13 +664,14 @@ void pg_pool_t::encode(bufferlist& bl, uint64_t features) const
     return;
   }
 
-  ENCODE_START(5, 5, bl);
+  ENCODE_START(6, 5, bl);
   ::encode(type, bl);
   ::encode(size, bl);
   ::encode(crush_ruleset, bl);
   ::encode(object_hash, bl);
   ::encode(pg_num, bl);
   ::encode(pgp_num, bl);
+  __u32 lpg_num = 0, lpgp_num = 0;  // tell old code that there are no localized pgs.
   ::encode(lpg_num, bl);
   ::encode(lpgp_num, bl);
   ::encode(last_change, bl);
@@ -631,15 +687,18 @@ void pg_pool_t::encode(bufferlist& bl, uint64_t features) const
 
 void pg_pool_t::decode(bufferlist::iterator& bl)
 {
-  DECODE_START_LEGACY_COMPAT_LEN(5, 5, 5, bl);
+  DECODE_START_LEGACY_COMPAT_LEN(6, 5, 5, bl);
   ::decode(type, bl);
   ::decode(size, bl);
   ::decode(crush_ruleset, bl);
   ::decode(object_hash, bl);
   ::decode(pg_num, bl);
   ::decode(pgp_num, bl);
-  ::decode(lpg_num, bl);
-  ::decode(lpgp_num, bl);
+  {
+    __u32 lpg_num, lpgp_num;
+    ::decode(lpg_num, bl);
+    ::decode(lpgp_num, bl);
+  }
   ::decode(last_change, bl);
   ::decode(snap_seq, bl);
   ::decode(snap_epoch, bl);
@@ -687,8 +746,6 @@ void pg_pool_t::generate_test_instances(list<pg_pool_t*>& o)
   a.object_hash = 4;
   a.pg_num = 6;
   a.pgp_num = 5;
-  a.lpg_num = 8;
-  a.lpgp_num = 7;
   a.last_change = 9;
   a.snap_seq = 10;
   a.snap_epoch = 11;
@@ -716,8 +773,6 @@ ostream& operator<<(ostream& out, const pg_pool_t& p)
       << " object_hash " << p.get_object_hash_name()
       << " pg_num " << p.get_pg_num()
       << " pgp_num " << p.get_pgp_num()
-      << " lpg_num " << p.get_lpg_num()
-      << " lpgp_num " << p.get_lpgp_num()
       << " last_change " << p.get_last_change()
       << " owner " << p.get_auid();
   if (p.flags)
@@ -1254,6 +1309,132 @@ void pg_info_t::generate_test_instances(list<pg_info_t*>& o)
 }
 
 
+// -- pg_interval_t --
+
+void pg_interval_t::encode(bufferlist& bl) const
+{
+  ENCODE_START(2, 2, bl);
+  ::encode(first, bl);
+  ::encode(last, bl);
+  ::encode(up, bl);
+  ::encode(acting, bl);
+  ::encode(maybe_went_rw, bl);
+  ENCODE_FINISH(bl);
+}
+
+void pg_interval_t::decode(bufferlist::iterator& bl)
+{
+  DECODE_START_LEGACY_COMPAT_LEN(2, 2, 2, bl);
+  ::decode(first, bl);
+  ::decode(last, bl);
+  ::decode(up, bl);
+  ::decode(acting, bl);
+  ::decode(maybe_went_rw, bl);
+  DECODE_FINISH(bl);
+}
+
+void pg_interval_t::dump(Formatter *f) const
+{
+  f->dump_unsigned("first", first);
+  f->dump_unsigned("last", last);
+  f->dump_int("maybe_went_rw", maybe_went_rw ? 1 : 0);
+  f->open_array_section("up");
+  for (vector<int>::const_iterator p = up.begin(); p != up.end(); ++p)
+    f->dump_int("osd", *p);
+  f->close_section();
+  f->open_array_section("acting");
+  for (vector<int>::const_iterator p = acting.begin(); p != acting.end(); ++p)
+    f->dump_int("osd", *p);
+  f->close_section();
+}
+
+void pg_interval_t::generate_test_instances(list<pg_interval_t*>& o)
+{
+  o.push_back(new pg_interval_t);
+  o.push_back(new pg_interval_t);
+  o.back()->up.push_back(1);
+  o.back()->acting.push_back(2);
+  o.back()->acting.push_back(3);
+  o.back()->first = 4;
+  o.back()->last = 5;
+  o.back()->maybe_went_rw = true;
+}
+
+bool pg_interval_t::check_new_interval(
+  const vector<int> &old_acting,
+  const vector<int> &new_acting,
+  const vector<int> &old_up,
+  const vector<int> &new_up,
+  epoch_t same_interval_since,
+  epoch_t last_epoch_clean,
+  OSDMapRef osdmap,
+  OSDMapRef lastmap,
+  map<epoch_t, pg_interval_t> *past_intervals,
+  std::ostream *out)
+{
+  // remember past interval
+  if (new_acting != old_acting || new_up != old_up) {
+    pg_interval_t& i = (*past_intervals)[same_interval_since];
+    i.first = same_interval_since;
+    i.last = osdmap->get_epoch() - 1;
+    i.acting = old_acting;
+    i.up = old_up;
+
+    if (i.acting.size()) {
+      if (lastmap->get_up_thru(i.acting[0]) >= i.first &&
+	  lastmap->get_up_from(i.acting[0]) <= i.first) {
+	i.maybe_went_rw = true;
+	if (out)
+	  *out << "generate_past_intervals " << i
+	       << " : primary up " << lastmap->get_up_from(i.acting[0])
+	       << "-" << lastmap->get_up_thru(i.acting[0])
+	       << std::endl;
+      } else if (last_epoch_clean >= i.first &&
+		 last_epoch_clean <= i.last) {
+	// If the last_epoch_clean is included in this interval, then
+	// the pg must have been rw (for recovery to have completed).
+	// This is important because we won't know the _real_
+	// first_epoch because we stop at last_epoch_clean, and we
+	// don't want the oldest interval to randomly have
+	// maybe_went_rw false depending on the relative up_thru vs
+	// last_epoch_clean timing.
+	i.maybe_went_rw = true;
+	if (out)
+	  *out << "generate_past_intervals " << i
+	       << " : includes last_epoch_clean " << last_epoch_clean
+	       << " and presumed to have been rw"
+	       << std::endl;
+      } else {
+	i.maybe_went_rw = false;
+	if (out)
+	  *out << "generate_past_intervals " << i
+	       << " : primary up " << lastmap->get_up_from(i.acting[0])
+	       << "-" << lastmap->get_up_thru(i.acting[0])
+	       << " does not include interval"
+	       << std::endl;
+      }
+    } else {
+      i.maybe_went_rw = false;
+      if (out)
+	*out << "generate_past_intervals " << i << " : empty" << std::endl;
+    }
+    return true;
+  } else {
+    return false;
+  }
+}
+
+ostream& operator<<(ostream& out, const pg_interval_t& i)
+{
+  out << "interval(" << i.first << "-" << i.last << " " << i.up << "/" << i.acting;
+  if (i.maybe_went_rw)
+    out << " maybe_went_rw";
+  out << ")";
+  return out;
+}
+
+
+
 // -- pg_query_t --
 
 void pg_query_t::dump(Formatter *f) const
@@ -1887,6 +2068,7 @@ void object_info_t::copy_user_bits(const object_info_t& other)
   truncate_size = other.truncate_size;
   lost = other.lost;
   category = other.category;
+  uses_tmap = other.uses_tmap;
 }
 
 ps_t object_info_t::legacy_object_locator_to_ps(const object_t &oid, 
@@ -1899,16 +2081,12 @@ ps_t object_info_t::legacy_object_locator_to_ps(const object_t &oid,
   else
     ps = ceph_str_hash(CEPH_STR_HASH_RJENKINS, oid.name.c_str(),
 		       oid.name.length());
-  // mix in preferred osd, so we don't get the same peers for
-  // all of the placement pgs (e.g. 0.0p*)
-  if (loc.get_preferred() >= 0)
-    ps += loc.get_preferred();
   return ps;
 }
 
 void object_info_t::encode(bufferlist& bl) const
 {
-  ENCODE_START(8, 8, bl);
+  ENCODE_START(9, 8, bl);
   ::encode(soid, bl);
   ::encode(oloc, bl);
   ::encode(category, bl);
@@ -1926,12 +2104,13 @@ void object_info_t::encode(bufferlist& bl) const
   ::encode(lost, bl);
   ::encode(watchers, bl);
   ::encode(user_version, bl);
+  ::encode(uses_tmap, bl);
   ENCODE_FINISH(bl);
 }
 
 void object_info_t::decode(bufferlist::iterator& bl)
 {
-  DECODE_START_LEGACY_COMPAT_LEN(8, 8, 8, bl);
+  DECODE_START_LEGACY_COMPAT_LEN(9, 8, 8, bl);
   if (struct_v >= 2 && struct_v <= 5) {
     sobject_t obj;
     ::decode(obj, bl);
@@ -1968,13 +2147,21 @@ void object_info_t::decode(bufferlist::iterator& bl)
     ::decode(watchers, bl);
     ::decode(user_version, bl);
   }
+  if (struct_v >= 9)
+    ::decode(uses_tmap, bl);
+  else
+    uses_tmap = true;
   DECODE_FINISH(bl);
 }
 
 void object_info_t::dump(Formatter *f) const
 {
-  f->dump_stream("oid") << soid;
-  f->dump_stream("locator") << oloc;
+  f->open_object_section("oid");
+  soid.dump(f);
+  f->close_section();
+  f->open_object_section("locator");
+  oloc.dump(f);
+  f->close_section();
   f->dump_string("category", category);
   f->dump_stream("version") << version;
   f->dump_stream("prior_version") << prior_version;
@@ -2022,6 +2209,138 @@ ostream& operator<<(ostream& out, const object_info_t& oi)
   return out;
 }
 
+// -- ObjectRecovery --
+void ObjectRecoveryProgress::encode(bufferlist &bl) const
+{
+  ENCODE_START(1, 1, bl);
+  ::encode(first, bl);
+  ::encode(data_complete, bl);
+  ::encode(data_recovered_to, bl);
+  ::encode(omap_recovered_to, bl);
+  ::encode(omap_complete, bl);
+  ENCODE_FINISH(bl);
+}
+
+void ObjectRecoveryProgress::decode(bufferlist::iterator &bl)
+{
+  DECODE_START(1, bl);
+  ::decode(first, bl);
+  ::decode(data_complete, bl);
+  ::decode(data_recovered_to, bl);
+  ::decode(omap_recovered_to, bl);
+  ::decode(omap_complete, bl);
+  DECODE_FINISH(bl);
+}
+
+ostream &operator<<(ostream &out, const ObjectRecoveryProgress &prog)
+{
+  return prog.print(out);
+}
+
+void ObjectRecoveryProgress::generate_test_instances(
+  list<ObjectRecoveryProgress*>& o)
+{
+  o.push_back(new ObjectRecoveryProgress);
+  o.back()->first = false;
+  o.back()->data_complete = true;
+  o.back()->omap_complete = true;
+  o.back()->data_recovered_to = 100;
+
+  o.push_back(new ObjectRecoveryProgress);
+  o.back()->first = true;
+  o.back()->data_complete = false;
+  o.back()->omap_complete = false;
+  o.back()->data_recovered_to = 0;
+}
+
+ostream &ObjectRecoveryProgress::print(ostream &out) const
+{
+  return out << "ObjectRecoveryProgress("
+	     << ( first ? "" : "!" ) << "first, "
+	     << "data_recovered_to:" << data_recovered_to
+	     << ", data_complete:" << ( data_complete ? "true" : "false" )
+	     << ", omap_recovered_to:" << omap_recovered_to
+	     << ", omap_complete:" << ( omap_complete ? "true" : "false" )
+	     << ")";
+}
+
+void ObjectRecoveryProgress::dump(Formatter *f) const
+{
+  f->dump_int("first?", first);
+  f->dump_int("data_complete?", data_complete);
+  f->dump_unsigned("data_recovered_to", data_recovered_to);
+  f->dump_int("omap_complete?", omap_complete);
+  f->dump_string("omap_recovered_to", omap_recovered_to);
+}
+
+void ObjectRecoveryInfo::encode(bufferlist &bl) const
+{
+  ENCODE_START(1, 1, bl);
+  ::encode(soid, bl);
+  ::encode(version, bl);
+  ::encode(size, bl);
+  ::encode(oi, bl);
+  ::encode(ss, bl);
+  ::encode(copy_subset, bl);
+  ::encode(clone_subset, bl);
+  ENCODE_FINISH(bl);
+}
+
+void ObjectRecoveryInfo::decode(bufferlist::iterator &bl)
+{
+  DECODE_START(1, bl);
+  ::decode(soid, bl);
+  ::decode(version, bl);
+  ::decode(size, bl);
+  ::decode(oi, bl);
+  ::decode(ss, bl);
+  ::decode(copy_subset, bl);
+  ::decode(clone_subset, bl);
+  DECODE_FINISH(bl);
+}
+
+void ObjectRecoveryInfo::generate_test_instances(
+  list<ObjectRecoveryInfo*>& o)
+{
+  o.push_back(new ObjectRecoveryInfo);
+  o.back()->soid = hobject_t(sobject_t("key", CEPH_NOSNAP));
+  o.back()->version = eversion_t(0,0);
+  o.back()->size = 100;
+}
+
+
+void ObjectRecoveryInfo::dump(Formatter *f) const
+{
+  f->dump_stream("object") << soid;
+  f->dump_stream("at_version") << version;
+  f->dump_stream("size") << size;
+  {
+    f->open_object_section("object_info");
+    oi.dump(f);
+    f->close_section();
+  }
+  {
+    f->open_object_section("snapset");
+    ss.dump(f);
+    f->close_section();
+  }
+  f->dump_stream("copy_subset") << copy_subset;
+  f->dump_stream("clone_subset") << clone_subset;
+}
+
+ostream& operator<<(ostream& out, const ObjectRecoveryInfo &inf)
+{
+  return inf.print(out);
+}
+
+ostream &ObjectRecoveryInfo::print(ostream &out) const
+{
+  return out << "ObjectRecoveryInfo("
+	     << soid << "@" << version
+	     << ", copy_subset: " << copy_subset
+	     << ", clone_subset: " << clone_subset
+	     << ")";
+}
 
 // -- ScrubMap --
 
